/**
 * inversify v.2.0.0-alpha.8 - A lightweight IoC container written in TypeScript.
 * Copyright (c) 2015 Remo H. Jansen
 * MIT inversify.io/LICENSE
 * http://inversify.io
 */
import Plan from "./plan";
import Context from "./context";
import Request from "./request";
import Target from "./target";
import * as METADATA_KEY from "../constants/metadata_keys";
import * as ERROR_MSGS from "../constants/error_msgs";
import BindingType from "../bindings/binding_type";
var Planner = (function () {
    function Planner() {
    }
    Planner.prototype.createContext = function (kernel) {
        return new Context(kernel);
    };
    Planner.prototype.createPlan = function (context, binding) {
        var _this = this;
        var rootRequest = new Request(binding.runtimeIdentifier, context, null, binding);
        var plan = new Plan(context, rootRequest);
        context.addPlan(plan);
        var dependencies = this._getDependencies(binding.implementationType);
        dependencies.forEach(function (target) { _this._createSubRequest(rootRequest, target); });
        return plan;
    };
    Planner.prototype.getBindings = function (kernel, service) {
        var bindings = [];
        var _kernel = kernel;
        var _bindingDictionary = _kernel._bindingDictionary;
        if (_bindingDictionary.hasKey(service)) {
            bindings = _bindingDictionary.get(service);
        }
        return bindings;
    };
    Planner.prototype._createSubRequest = function (parentRequest, target) {
        try {
            var bindings = this.getBindings(parentRequest.parentContext.kernel, target.service);
            var activeBindings = [];
            if (bindings.length > 1 && target.isArray() === false) {
                activeBindings = bindings.filter(function (binding) {
                    var request = new Request(binding.runtimeIdentifier, parentRequest.parentContext, parentRequest, binding, target);
                    return binding.constraint(request);
                });
            }
            else {
                activeBindings = bindings;
            }
            if (activeBindings.length === 0) {
                throw new Error(ERROR_MSGS.NOT_REGISTERED + " " + target.getServiceAsString());
            }
            else if (activeBindings.length > 1 && target.isArray() === false) {
                throw new Error(ERROR_MSGS.AMBIGUOUS_MATCH + " " + target.getServiceAsString());
            }
            else {
                this._createChildRequest(parentRequest, target, activeBindings);
            }
        }
        catch (error) {
            if (error instanceof RangeError) {
                this._throwWhenCircularDependenciesFound(parentRequest.parentContext.plan.rootRequest);
            }
            else {
                throw new Error(error.message);
            }
        }
    };
    Planner.prototype._createChildRequest = function (parentRequest, target, bindings) {
        var _this = this;
        var childRequest = parentRequest.addChildRequest(target.service, bindings, target);
        var subChildRequest = childRequest;
        bindings.forEach(function (binding) {
            if (target.isArray()) {
                subChildRequest = childRequest.addChildRequest(binding.runtimeIdentifier, binding, target);
            }
            if (binding.type === BindingType.Instance) {
                var subDependencies = _this._getDependencies(binding.implementationType);
                subDependencies.forEach(function (d, index) {
                    _this._createSubRequest(subChildRequest, d);
                });
            }
        });
    };
    Planner.prototype._throwWhenCircularDependenciesFound = function (request, previousServices) {
        var _this = this;
        if (previousServices === void 0) { previousServices = []; }
        previousServices.push(request.service);
        request.childRequests.forEach(function (childRequest) {
            var service = childRequest.service;
            if (previousServices.indexOf(service) === -1) {
                if (childRequest.childRequests.length > 0) {
                    _this._throwWhenCircularDependenciesFound(childRequest, previousServices);
                }
                else {
                    previousServices.push(service);
                }
            }
            else {
                throw new Error(ERROR_MSGS.CIRCULAR_DEPENDENCY + " " + service + " and " + request.service);
            }
        });
    };
    Planner.prototype._getDependencies = function (func) {
        if (func === null) {
            return [];
        }
        var targetsTypes = Reflect.getMetadata(METADATA_KEY.PARAM_TYPES, func);
        if (targetsTypes === undefined) {
            var constructorName = func.name;
            var msg = ERROR_MSGS.MISSING_INJECTABLE_ANNOTATION + " " + constructorName + ".";
            throw new Error(msg);
        }
        var targetsMetadata = Reflect.getMetadata(METADATA_KEY.TAGGED, func) || [];
        var targets = targetsTypes.map(function (targetType, index) {
            var targetMetadata = targetsMetadata[index.toString()] || [];
            var targetMetadataMap = {};
            targetMetadata.forEach(function (m) {
                targetMetadataMap[m.key.toString()] = m.value;
            });
            var inject = targetMetadataMap[METADATA_KEY.INJECT_TAG];
            var multiInject = targetMetadataMap[METADATA_KEY.MULTI_INJECT_TAG];
            var targetName = targetMetadataMap[METADATA_KEY.NAME_TAG];
            targetType = (inject || multiInject) ? (inject || multiInject) : targetType;
            if (targetType === Object || targetType === Function) {
                var constructorName = func.name;
                var msg = ERROR_MSGS.MISSING_INJECT_ANNOTATION + " argument " + index + " in class " + constructorName + ".";
                throw new Error(msg);
            }
            var target = new Target(targetName, targetType);
            target.metadata = targetMetadata;
            return target;
        });
        return targets;
    };
    return Planner;
}());
export default Planner;
