/**
 * inversify v.2.0.0-alpha.8 - A lightweight IoC container written in TypeScript.
 * Copyright (c) 2015 Remo H. Jansen
 * MIT inversify.io/LICENSE
 * http://inversify.io
 */
"use strict";
var binding_count_1 = require("../bindings/binding_count");
var binding_1 = require("../bindings/binding");
var lookup_1 = require("./lookup");
var planner_1 = require("../planning/planner");
var resolver_1 = require("../resolution/resolver");
var ERROR_MSGS = require("../constants/error_msgs");
var binding_to_syntax_1 = require("../syntax/binding_to_syntax");
var Kernel = (function () {
    function Kernel() {
        this._planner = new planner_1.default();
        this._resolver = new resolver_1.default();
        this._bindingDictionary = new lookup_1.default();
        this._middleware = null;
    }
    Kernel.prototype.load = function () {
        var _this = this;
        var modules = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            modules[_i - 0] = arguments[_i];
        }
        modules.forEach(function (module) { module(_this); });
    };
    Kernel.prototype.applyMiddleware = function () {
        var middlewares = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            middlewares[_i - 0] = arguments[_i];
        }
        this._middleware = middlewares.reverse().reduce(function (prev, curr) {
            return curr(prev);
        }, this._resolver.resolve.bind(this._resolver));
    };
    Kernel.prototype.bind = function (runtimeIdentifier) {
        var binding = new binding_1.default(runtimeIdentifier);
        this._bindingDictionary.add(runtimeIdentifier, binding);
        return new binding_to_syntax_1.default(binding);
    };
    Kernel.prototype.unbind = function (runtimeIdentifier) {
        try {
            this._bindingDictionary.remove(runtimeIdentifier);
        }
        catch (e) {
            throw new Error(ERROR_MSGS.CANNOT_UNBIND + " " + runtimeIdentifier);
        }
    };
    Kernel.prototype.unbindAll = function () {
        this._bindingDictionary = new lookup_1.default();
    };
    Kernel.prototype.get = function (runtimeIdentifier) {
        var bindings = this._planner.getBindings(this, runtimeIdentifier);
        switch (bindings.length) {
            case binding_count_1.default.NoBindingsAvailable:
                throw new Error(ERROR_MSGS.NOT_REGISTERED + " " + runtimeIdentifier);
            case binding_count_1.default.OnlyOneBindingAvailable:
                return this._planAndResolve(bindings[0]);
            case binding_count_1.default.MultipleBindingsAvailable:
            default:
                throw new Error(ERROR_MSGS.AMBIGUOUS_MATCH + " " + runtimeIdentifier);
        }
    };
    Kernel.prototype.getAll = function (runtimeIdentifier) {
        var _this = this;
        var bindings = this._planner.getBindings(this, runtimeIdentifier);
        switch (bindings.length) {
            case binding_count_1.default.NoBindingsAvailable:
                throw new Error(ERROR_MSGS.NOT_REGISTERED + " " + runtimeIdentifier);
            case binding_count_1.default.OnlyOneBindingAvailable:
            case binding_count_1.default.MultipleBindingsAvailable:
            default:
                return bindings.map(function (binding) {
                    return _this._planAndResolve(binding);
                });
        }
    };
    Kernel.prototype._planAndResolve = function (binding) {
        var context = this._planner.createContext(this);
        this._planner.createPlan(context, binding);
        return (this._middleware !== null) ? this._middleware(context) : this._resolver.resolve(context);
    };
    return Kernel;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Kernel;
